"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var vqb = require("../");
var assert = require("assert");
exports.DEFAULT_OP = "equal";
exports.BSONTypes = [
    { value: "double", text: "Double", number: 1 },
    { value: "string", text: "String", number: 2 },
    { value: "object", text: "Object", number: 3 },
    { value: "array", text: "Array", number: 4 },
    { value: "binData", text: "Binary data", number: 5 },
    { value: "undefined", text: "Undefined", number: 6 },
    { value: "objectId", text: "ObjectId", number: 7 },
    { value: "bool", text: "Boolean", number: 8 },
    { value: "date", text: "Date", number: 9 },
    { value: "null", text: "Null", number: 10 },
    { value: "regex", text: "Regular Expression", number: 11 },
    { value: "dbPointer", text: "DBPointer", number: 12 },
    { value: "javascript", text: "JavaScript", number: 13 },
    { value: "symbol", text: "Symbol", number: 14 },
    { value: "javascriptWithScope", text: "JavaScript (with scope)", number: 15 },
    { value: "int", text: "32-bit integer", number: 16 },
    { value: "timestamp", text: "Timestamp", number: 17 },
    { value: "decimal", text: "Decimal", number: 18 },
    { value: "long", text: "64-bit integer", number: 19 },
    { value: "minKey", text: "Min key", number: -1 },
    { value: "maxKey", text: "Max key", number: 127 },
];
function findRuleOp(op) {
    var ruleOp = _.find(exports.operators, function (it) { return it.value === op; });
    if (_.isUndefined(ruleOp.types)) {
        ruleOp.types = vqb.getDataTypeList().map(function (it) {
            return { fieldType: it };
        });
    }
    return ruleOp;
}
exports.findRuleOp = findRuleOp;
function inOrNotInMongoOp(val, op) {
    var vals = vqb.takeOffQuoted(val).split(",").map(function (it) { return it.trim(); }).map(function (it) {
        if (vqb.isValidVar(it))
            return "\"" + it + "\"";
        else {
            if (/^(\"|\')(.*)(\"|\')$/.test(it)) {
                return it;
            }
            return vqb.takeOnQuoted("" + it);
        }
    }).join(",");
    var rst = {};
    rst[op] = [vqb.takeOnQuoted(vals)];
    return rst;
}
var NumericTypes = [{ fieldType: "Number" }, { fieldType: "Int64" }, { fieldType: "Decimal" }, { fieldType: "Date" }, { fieldType: "DateTime" }, { fieldType: "Any" }];
var StringTypes = [{ fieldType: "String" }];
var getRegexOpVal = function (val, type) {
    if (type === "String")
        return "\"" + val + "\"";
    assert(type === "Any");
    if (vqb.isValidVar(val)) {
        return "\"" + val + "\"";
    }
    else
        return val;
};
exports.operators = [
    { value: "equal", mongoOp: "" },
    { value: "not equal", mongoOp: "$ne" },
    {
        value: "contains", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.contains(" + getRegexOpVal(val, type) + ")");
        }
    },
    {
        value: "not contains", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.notContains(" + getRegexOpVal(val, type) + ")");
        }
    },
    {
        value: "starts with", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.startsWith(" + getRegexOpVal(val, type) + ")");
        }
    },
    {
        value: "not starts with", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.notStartsWith(" + getRegexOpVal(val, type) + ")");
        }
    },
    {
        value: "ends with", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.endsWith(" + getRegexOpVal(val, type) + ")");
        }
    },
    {
        value: "not ends with", types: StringTypes, mongoOp: function (val, type) {
            return vqb.takeOnQuoted("mb.regex.notEndsWith(" + getRegexOpVal(val, type) + ")");
        }
    },
    { value: "less", types: NumericTypes, mongoOp: "$lt" },
    { value: "less or equal", types: NumericTypes, mongoOp: "$lte" },
    { value: "greater", types: NumericTypes, mongoOp: "$gt" },
    { value: "greater or equal", types: NumericTypes, mongoOp: "$gte" },
    // {value:"between(<=..<=)"},
    // {value:"not between"},
    {
        value: "in", types: [{ fieldType: "Any", editorType: "MultiValues" }], mongoOp: function (val) {
            return inOrNotInMongoOp(val, '$in');
        }
    },
    { value: "not in", types: [{ fieldType: "Any", editorType: "MultiValues" }], mongoOp: function (val) {
            return inOrNotInMongoOp(val, '$nin');
        } },
    { value: "is empty", types: StringTypes, noValue: true, mongoOp: function () {
            return "";
        } },
    { value: "is not empty", types: StringTypes, noValue: true, mongoOp: function () {
            return { $ne: "" };
        } },
    { value: "is null", types: [], noValue: true, mongoOp: function () {
            return null;
        } },
    { value: "is not null", types: [], noValue: true, mongoOp: function () {
            return { $ne: null };
        } },
    { value: "exists", types: [], noValue: true, mongoOp: function () {
            return { $exists: true };
        } },
    { value: "not exists", types: [], noValue: true, mongoOp: function () {
            return { $exists: false };
        } },
    { value: "has type", types: [{ fieldType: "String" }], perserveValue: false, allowedValues: exports.BSONTypes, mongoOp: "$type" },
    { value: "text search", types: [{ fieldType: "String", editorType: "TextSearch" }], noField: true, mongoOp: function (val) {
            return { $text: { $search: val } };
        } },
    { value: "(custom)", types: [{ fieldType: "Any" }], perserveValue: false, mongoOp: function (val) {
            return val;
        } },
];
//operators=["=","<>", ">","<",">=","<="];
//# sourceMappingURL=RuleOperator.js.map