"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var vqb = require("../");
var _ = require("lodash");
var moment = require("moment");
var assert = require("assert");
exports.Mquery_ext_methods = ["all",
    "and",
    "box",
    "circle",
    "elemMatch",
    'eq',
    "equals",
    "exists",
    "geometry",
    "gt",
    "gte",
    "in",
    "intersects",
    "lt",
    "lte",
    "maxDistance",
    "mod",
    "ne",
    "nin",
    "nor",
    "near",
    "or",
    "polygon",
    "regex",
    "select",
    "selected",
    "selectedInclusively",
    "selectedExclusively",
    "$size",
    "slice",
    "within",
    //add by qinghai, 40
    "not",
    "type",
    //"text",
    "bitsAllSet", "bitsAnySet", "bitsAllClear", "bitsAnyClear"
];
var TAKEON_MAGIC = "@&@&@&@";
function takeOnQuoted(str) {
    return "" + TAKEON_MAGIC + str + TAKEON_MAGIC;
}
exports.takeOnQuoted = takeOnQuoted;
function takeOffQuoted(str, inJson) {
    if (inJson === void 0) { inJson = true; }
    if (inJson)
        str = str.replace(new RegExp("\"" + TAKEON_MAGIC + "(.*?)" + TAKEON_MAGIC + "\"", "g"), "$1");
    return str.replace(new RegExp(TAKEON_MAGIC, "g"), "");
}
exports.takeOffQuoted = takeOffQuoted;
function getGroupMongoQuery(qry) {
    var self = this;
    var cond = qry.condition.toLowerCase();
    var rules = qry.rules || [];
    var ruleQrys = rules.map(function (it) {
        if (it.condition) {
            return getGroupMongoQuery(it);
        }
        else {
            return getRuleMongoQuery(it);
        }
    });
    var rst = (function () {
        if (ruleQrys.length <= 1) {
            return ruleQrys && ruleQrys[0];
        }
        if (cond === "or") {
            return { $or: ruleQrys };
        }
        assert(cond === "and");
        var hasDupOp = _.intersection.apply(self, ruleQrys.map(function (it) { return _.keys(it).filter(function (it) { return it[0] === "$"; }); })).length > 0;
        if (hasDupOp) {
            return { $and: ruleQrys };
        }
        return _.merge.apply({}, ruleQrys);
    })();
    rst = rst || {};
    if (qry.isNot) {
        rst = {
            "$nor": [
                __assign({}, rst)
            ]
        };
    }
    return rst;
}
exports.getGroupMongoQuery = getGroupMongoQuery;
function isValidVar(val) {
    var varVal = require('var-validator');
    varVal.enableScope = false;
    varVal.enableBrackets = false;
    return varVal.isValid(val);
}
exports.isValidVar = isValidVar;
function parseDate(v, dateOnly) {
    if (!dateOnly) {
        var m = moment(v, vqb.DATE_TIME_PATTERN);
        if (m.isValid())
            return m;
        m = moment(v, "YYYY-MM-DD hh:mm:ss");
        if (m.isValid())
            return m;
    }
    else {
        var m = moment(v, vqb.DATE_PATTERN);
        if (m.isValid())
            return m;
        m = moment(v, "YYYY-MM-DD");
        if (m.isValid())
            return m;
    }
    return moment(v);
}
function getRuleMongoQuery(params) {
    var op = vqb.findRuleOp(params.operator);
    assert(op);
    var rst = {};
    var value = (function () {
        var rst = params.value;
        if (_.isUndefined(rst))
            return "";
        if (rst === "")
            return "";
        if (params.type === "Boolean") {
            return vqb.valueToBoolean(params.value);
        }
        if (params.type === "Date") {
            return vqb.takeOnQuoted("ISODate(\"" + parseDate(params.value, true).toISOString() + "\")");
        }
        if (params.type === "DateTime") {
            return vqb.takeOnQuoted("ISODate(\"" + parseDate(params.value, false).toISOString() + "\")");
        }
        if (params.type === "Number") {
            var val = (function (val) {
                if (_.isNumber(val))
                    return val;
                if (_.isString(val)) {
                    return parseFloat(val.replace(/\,/g, ""));
                }
                return val;
            })(params.value);
            return val;
        }
        if (params.type === "Int64") {
            return vqb.takeOnQuoted("NumberLong(\"" + params.value.replace(/\,/g, "") + "\")");
        }
        if (params.type === "Decimal") {
            return vqb.takeOnQuoted("NumberDecimal(\"" + params.value.replace(/\,/g, "") + "\")");
        }
        if (params.type === "ObjectId") {
            return vqb.takeOnQuoted("" + params.value);
        }
        if (params.type === "Any") {
            if (isValidVar(params.value))
                return params.value;
            else
                return vqb.takeOnQuoted("" + params.value);
        }
        if ((params.operator === "has type") && vqb.options.convertBsonTypeToNumber) {
            var typeV = _.find(vqb.BSONTypes, (function (it) {
                var str = (rst || "").toString();
                return (it.value === str) || (it.number.toString() === str);
            }));
            if (typeV) {
                return typeV.number;
            }
        }
        return rst;
    })();
    if (op.mongoOp === undefined) {
    }
    else {
        if (_.isString(op.mongoOp)) {
            if (_.isEmpty(op.mongoOp)) {
                rst[params.field] = value;
            }
            else {
                var obj = {};
                obj[op.mongoOp] = value;
                rst[params.field] = obj;
            }
        }
        else {
            assert(_.isFunction(op.mongoOp));
            var opRst = op.mongoOp(value, params.type);
            if (op.noField) {
                rst = opRst;
            }
            else {
                rst[params.field] = opRst;
            }
        }
    }
    return rst;
}
exports.getRuleMongoQuery = getRuleMongoQuery;
function isSingleKeySimpleObj(v) {
    if (_.isObject(v) && !_.isArray(v)) {
        var keys = _.keys(v);
        if (keys.length !== 1)
            return false;
        return !_.isObjectLike(v[keys[0]]);
    }
    return false;
}
//todo: test custom
function doTransFluent(mongoQry) {
    if (_.isArray(mongoQry)) {
        var hasBuildPart_1 = false;
        var qryAr_1 = mongoQry.map(function (it) {
            if (isSingleKeySimpleObj(it)) {
                var key0 = _.keys(it)[0];
                var key = isValidVar(key0) ? key0 : "\"" + key0 + "\"";
                hasBuildPart_1 = true;
                if (_.isString(it[key0])) {
                    return "{" + key + ":\"" + it[key0] + "\"}";
                }
                return "{" + key + ": " + vqb.stringify(it[key0], vqb.fakeMongo.takeOnQuotedStringifyReplacer) + "}";
            }
            var str = doTransFluent(it);
            if (_.isObject(it) && !_.isArray(it)) {
                if (/^(\n)?\./.test(str)) {
                    hasBuildPart_1 = true;
                    return "qb" + str + ".build()";
                }
            }
            return str;
        });
        var canCollpase = (function () {
            if (qryAr_1.length !== 1)
                return false;
            return hasBuildPart_1 || qryAr_1[0] === "{}";
        })();
        if (canCollpase) {
            return qryAr_1[0];
        }
        else {
            return "[" + qryAr_1.join(",") + "]";
        }
    }
    if (_.isObject(mongoQry)) {
        return handleObj(mongoQry);
    }
    if (_.isString(mongoQry)) {
        return "\"" + mongoQry + "\"";
    }
    return mongoQry;
    function handleObj(obj) {
        if (_.isEmpty(obj))
            return "{}";
        var keys = _.keys(obj);
        // if (keys.length===1 &&  !_.isObjectLike(obj[keys[0]])){
        //   let key0=keys[0];
        //   return `{${key0}:${obj[key0]}}`;
        // }
        return _.reduce(keys, function (memo, k) {
            var v = obj[k];
            var part = "";
            if (k[0] === "$") {
                var op = k.slice(1);
                var $norAndOr = ["$nor", "$and", "$or"];
                if ($norAndOr.indexOf(k) > -1) {
                    part = "." + op + "(" + doTransFluent(v) + ")";
                    //part = `qb.${op}(${doTransFluent(v)}).build()`;
                }
                else if (exports.Mquery_ext_methods.indexOf(op) > -1) {
                    part = "." + op + "(" + doTransFluent(v) + ")";
                }
                else {
                    part = "\n.where(\"" + k + "\"," + vqb.stringify(v, vqb.fakeMongo.takeOnQuotedStringifyReplacer) + ")";
                }
            }
            else {
                part = "\n.where(\"" + k + "\")" + (_.isObject(v) ? doTransFluent(v) : ".eq(" + doTransFluent(v) + ")");
            }
            return memo + vqb.takeOffQuoted(part);
        }, "");
    }
}
function transformMongoQueryToFluent(mongoQry) {
    if (_.isEmpty(mongoQry))
        return "";
    var str = doTransFluent(mongoQry);
    str = vqb.takeOffQuoted(str, false);
    str = str.replace(/^qb\.([^]*)\.build\(\)$/m, ".$1"); //cross multi-lines
    str = str.replace(/qb\n\.where/g, "qb.where");
    str = str.replace(/\bqb\./g, "\nqb.");
    str = str.replace(/\.build\(\)\]/g, ".build()\n]");
    str = str.replace(/^(\n| )*/g, "");
    return str;
}
function stringify(obj_from_json, replacer, space) {
    if (_.isArray(obj_from_json)) {
        return "[" + obj_from_json.map(function (it) { return stringify(it); }).join(",") + "]";
    }
    if (!_.isObject(obj_from_json)) {
        // not an object, stringify using native function
        return JSON.stringify(obj_from_json, replacer, space);
    }
    if (obj_from_json && obj_from_json.toJSON && _.isFunction(obj_from_json.toJSON)) {
        return "\"" + obj_from_json.toJSON() + "\"";
    }
    // Implements recursive object serialization according to JSON spec
    // but without quotes around the keys.
    var props = Object
        .keys(obj_from_json)
        .map(function (key) {
        var val = stringify(obj_from_json[key], replacer, space);
        //val=_.isString(obj_from_json[key])?`"${val}"`:val;
        return (vqb.isValidVar(key) ? key : "\"" + key + "\"") + ":" + val;
    })
        .join(",");
    return "{" + props + "}";
}
exports.stringify = stringify;
function getMongoQueryAsString(grp) {
    return vqb.takeOffQuoted(stringify(vqb.getGroupMongoQuery(grp), null, 4)).replace(/\\\"/g, '"');
}
exports.getMongoQueryAsString = getMongoQueryAsString;
function getFluentMongoQueryString(grp) {
    return transformMongoQueryToFluent(vqb.getGroupMongoQuery(grp));
}
exports.getFluentMongoQueryString = getFluentMongoQueryString;
//# sourceMappingURL=mongoQuery.js.map