"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var vqb = require("../");
var _ = require("lodash");
var assert = require("assert");
var moment = require("moment");
function getValueAsString(it) {
    if (_.isUndefined(it))
        return "undefined";
    if (_.isString(it))
        return it;
    if (_.isRegExp(it)) {
        return it.toString();
    }
    var getDateValAsString = function (v) {
        if (isDateOnlyStr(v)) {
            return moment(v).format(vqb.DATE_PATTERN);
        }
        else
            return moment(v).format(vqb.DATE_TIME_PATTERN);
    };
    if (_.isDate(it)) {
        return getDateValAsString(it);
    }
    if (it instanceof vqb.fakeMongo.FakeMongoToJson) {
        if (it.bsonType === "ISODate") {
            var v = it.params[0] || "";
            //console.log("ISODate", v);
            return getDateValAsString(v);
        }
        if (["NumberInt", "NumberLong", "NumberDecimal"].indexOf(it.bsonType) > -1) {
            return it.params[0] || "";
        }
        else
            return it.toString();
    }
    return JSON.stringify(it);
}
function isDateOnlyStr(v) {
    return _.isEqual(moment(v), moment(v).startOf(("day")));
}
function getValueType(v) {
    if (_.isUndefined(v))
        return "Any";
    if (_.isNumber(v))
        return "Number";
    if (_.isString(v))
        return "String";
    if (_.isBoolean(v))
        return "Boolean";
    var getDateType = (function (v) {
        if (isDateOnlyStr(v)) {
            return "Date";
        }
        else
            return "DateTime";
    });
    if (_.isDate(v)) {
        return getDateType(v);
    }
    if (v instanceof vqb.fakeMongo.FakeMongoToJson) {
        var type = v.bsonType;
        if (type === "ObjectId")
            return "ObjectId";
        if (type === "NumberInt")
            return "Number";
        if (type === "NumberLong")
            return "Int64";
        if (type === "NumberLong")
            return "Int64";
        if (type === "NumberDecimal")
            return "Decimal";
        if (type === "ISODate") {
            var p0 = v.params[0];
            return getDateType(p0);
        }
        return "Any";
        // Date: DateRuleValueEditor,
        // DateTime: DateTimeRuleValueEditor,
        // UUID:{editor:InputMaskRuleValueEditor, inputmask:`UUID("*{8}-*{4}-*{4}-*{4}-*{12}")`},
        // LUUID:{editor:InputMaskRuleValueEditor, inputmask:`LUUID("*{8}-*{4}-*{4}-*{4}-*{12}")`},
        // MD5: {editor:InputMaskRuleValueEditor, inputmask:`md5("*{32}")`},
        // Symbol:  {editor:InputMaskRuleValueEditor, inputmask:`Symbol("*{0,1024}")`},
        // Regex: {editor:InputMaskRuleValueEditor, inputmask:"/*{0,20548}/*{0,6}"},
    }
    return "Any";
}
function handleNorOp(k, v) {
    assert(k.toLowerCase() === "$nor");
    var val = _.isArray(v) ? v : [v];
    return val.map(function (it) {
        var rst = parseMongoQuery(it, true);
        rst.isNot = true;
        return rst;
    });
}
function handleAndOrOp(k, v) {
    var ruleList = (function () {
        var vAr = _.isArray(v) ? v : [v];
        return vAr.map(function (it) {
            var rst = parseMongoQuery(it, false);
            if (_.isArray(rst) && (rst.length === 1)) {
                rst = rst[0];
            }
            return rst;
        });
    })();
    var flatRuleList = (function () {
        var rst = [];
        ruleList.forEach(function (it) {
            if (!_.isArray(it)) {
                return rst.push(it);
            }
            if (it.length === 1) {
                return rst.push(it[0]);
            }
            var rule = {
                condition: "AND",
                rules: it
            };
            rst.push(rule);
        });
        return rst;
        //if (k==="$and") return _.flattenDeep(ruleList)
    })();
    var andRst = {
        condition: k.slice(1).toUpperCase(),
        rules: flatRuleList,
    };
    return andRst;
}
function collapseRules(rules, isTopLevel) {
    rules = rules || [];
    if (rules && (_.isArray(rules[0])) && (rules.length === 1)) {
        rules = rules[0];
    }
    if (!isTopLevel) {
        if (rules.length === 1) {
            return rules[0];
        }
        return rules;
    }
    else {
        if (rules.length === 1) {
            if (rules[0].condition) {
                return rules[0];
            }
        }
        //console.log("flat rules", rules);
        return {
            "condition": "AND",
            "rules": _.flattenDeep(rules)
        };
    }
}
var mnogoOpRuleOpMap = {
    "$eq": "equal",
    "$ne": "not equal",
    // :"contains",
    // :"not contains",
    // :"starts with",
    // :"not starts with",
    // :"ends with",
    // :"not ends with",
    "$lt": "less",
    "$lte": "less or equal",
    "$gt": "greater",
    "$gte": "greater or equal",
};
function handleNamedRule(key, value) {
    var field = key;
    var is$op = (function () {
        if (!_.isPlainObject(value))
            return false;
        if (_.isArray(value))
            return false;
        return !_.isEmpty(_.find(_.keys(value), function (it) { return it[0] === "$"; }));
    })();
    if (is$op) {
        var rules = _.keys(value).map(function (op) {
            var lk = op.toLowerCase();
            var val = value[op];
            if ((op === "$eq" || op === "$equal") && val === "") {
                return { field: field, operator: "is empty", value: "", type: "String" };
            }
            if (op === "$ne" && val === "") {
                return { field: field, operator: "is not empty", value: "", type: "String" };
            }
            if ((op === "$eq" || op === "$equal") && val === null) {
                return { field: field, operator: "is null", value: "", type: "String" };
            }
            if (op === "$ne" && val === null) {
                return { field: field, operator: "is not null", value: "", type: "String" };
            }
            if (op === "$exists") {
                if (!!val) {
                    return { field: field, operator: "exists", value: "", type: "String" };
                }
                else {
                    return { field: field, operator: "not exists", value: "", type: "String" };
                }
            }
            if (op === "$type") {
                var typeV = _.find(vqb.BSONTypes, (function (it) {
                    var str = (val || "").toString();
                    return (it.value === str) || (it.number.toString() === str);
                }));
                return { field: field, operator: "has type", value: (typeV && typeV.value) || "", type: "String" };
            }
            var getArrayValue = function (val) {
                if (!_.isArray(val))
                    return getValueAsString(val);
                return val.map(function (it) { return _.isString(it) ? "\"" + it + "\"" : it; }).join(",");
            };
            if (op === "$in") {
                return { field: field, operator: "in", value: getArrayValue(val), type: "Any" };
            }
            if (op === "$nin") {
                return { field: field, operator: "not in", value: getArrayValue(val), type: "Any" };
            }
            var ruleOp = mnogoOpRuleOpMap[op];
            if (ruleOp) {
                return { field: field, operator: ruleOp, value: getValueAsString(value[op]), type: getValueType(value[op]) };
            }
            var tmpV = value[op];
            var customVal = _.isString(tmpV) ? "\"" + tmpV + "\"" : getValueAsString(tmpV);
            return { field: field, operator: "(custom)", value: "{\"" + op + "\": " + customVal + "}", type: "Any" };
        });
        if (rules.length === 1) {
            rules = rules[0];
        }
        return rules;
    }
    else {
        return handleNamedRule(key, { $eq: value });
    }
}
function parseMongoQuery(obj, isTopLevel) {
    if (isTopLevel === void 0) { isTopLevel = true; }
    assert(_.isObject(obj));
    var keys = _.keys(obj);
    if (_.find(keys.map(function (it) { return it.toLowerCase(); }), function (it) { return ["$nor", "$or"].indexOf(it) > -1; })) {
        keys = keys.sort(function (a, b) {
            if (b === "$nor" || b === "$or")
                return -1;
            if (a === "$nor" || a === "$or")
                return 1;
            return 0;
        });
    }
    var rules = keys.map(function (k) {
        var v = obj[k];
        //for contains, not contains ...
        if (v instanceof vqb.fakeMongo.FakeMongoOpRuleClass) {
            return { field: k, operator: v.op, value: v.value, type: v.type };
        }
        var lk = k.toLowerCase();
        if (lk === "$nor") {
            return handleNorOp(lk, v);
        }
        if (["$and", "$or"].indexOf(lk) > -1) {
            return handleAndOrOp(lk, v);
        }
        if (k[0] === "$") {
            if (k === "$text" && _.isEqual(_.keys(v), ["$search"])) {
                return {
                    "field": "(placeholder)",
                    "operator": "text search",
                    "value": getValueAsString(v["$search"]),
                    "type": getValueType(v["$search"]),
                };
            }
            else {
                return {
                    "field": k,
                    "operator": "equal",
                    "value": getValueAsString(v),
                    "type": getValueType(v)
                };
            }
        }
        var rst = handleNamedRule(k, v);
        return rst;
    });
    return collapseRules(rules, isTopLevel);
}
exports.parseMongoQuery = parseMongoQuery;
//# sourceMappingURL=parseMongoQuery.js.map